from botocore.exceptions import ClientError
from navalmartin_mir_aws_utils import (
    get_aws_client_factory,
    AWSCredentials_SecretsManager,
)

AWS_SECRET_NAME = "YOUR-AWS-Secrets-Manager"
AWS_REGION = "YOUR-AWS-REGION"

if __name__ == "__main__":
    try:
        credentials = AWSCredentials_SecretsManager(
            aws_region=AWS_REGION, secret_name=AWS_SECRET_NAME
        )
        print(credentials)
        client = get_aws_client_factory(credentials=credentials)

        get_secret_value_response = client.get_secret_value(
            SecretId=credentials.secret_name
        )
        print(get_secret_value_response)
    except ClientError as e:
        print(str(e))



from typing import Any
from navalmartin_mir_aws_utils.aws_credentials import AWSCredentials_SQS
from navalmartin_mir_aws_utils.boto3_client import get_aws_sqs_client
from navalmartin_mir_aws_utils.sqs_queue_config import SQSMessageConfig


def send_sqs_message(
    sqs_credentials: AWSCredentials_SQS,
    sqs_msg: SQSMessageConfig,
    sqs_client: Any = None,
) -> dict:
    """Send a message to the SQS queue specified
    under the sqs credentials

    Parameters
    ----------
    sqs_credentials: The credentials to use to send the message
    sqs_msg: The message configuration

    Returns
    -------
    A dictionary with the response attributes
    """

    if sqs_client is not None:
        response = sqs_client.send_message(
            QueueUrl=sqs_credentials.queue_url,
            MessageBody=sqs_msg.message_body,
            # DelaySeconds=sqs_msg.delay_seconds,
            MessageAttributes=sqs_msg.message_attributes
            if sqs_msg.message_attributes is not None
            else {},
            MessageGroupId=sqs_msg.message_group_id,
            MessageDeduplicationId=sqs_msg.message_deduplication_id,
        )
        return response

    new_sqs_client = get_aws_sqs_client(credentials=sqs_credentials)
    response: dict = new_sqs_client.send_message(
        QueueUrl=sqs_credentials.queue_url,
        MessageBody=sqs_msg.message_body,
        # DelaySeconds=sqs_msg.delay_seconds,
        MessageAttributes=sqs_msg.message_attributes
        if sqs_msg.message_attributes is not None
        else {},
        MessageGroupId=sqs_msg.message_group_id,
        MessageDeduplicationId=sqs_msg.message_deduplication_id,
    )
    return response


def read_one_sqs_message(
    sqs_credentials: AWSCredentials_SQS, sqs_client: Any = None
) -> dict:
    """

    Parameters
    ----------
    sqs_client: The SQS client to use for the operation
    sqs_credentials: The credentials to use for the queue

    Returns
    -------

    A dictionary of the response
    """

    if sqs_client is not None:
        response = sqs_client.receive_message(
            QueueUrl=sqs_credentials.queue_url, MaxNumberOfMessages=1
        )
        return response

    new_sqs_client = get_aws_sqs_client(credentials=sqs_credentials)
    response = new_sqs_client.receive_message(
        QueueUrl=sqs_credentials.queue_url, MaxNumberOfMessages=1
    )

    return response


def delete_sqs_message(
    sqs_credentials: AWSCredentials_SQS, receipt_handle: str, sqs_client: Any = None
) -> dict:
    """Deletes the message from the queue

    Parameters
    ----------
    sqs_client: The SQS client to use for the operation
    sqs_credentials: The credentials to use for the queue
    receipt_handle: This is the handle received when receiving the message

    Returns
    -------

    """

    if sqs_client is not None:
        response = sqs_client.delete_message(
            QueueUrl=sqs_credentials.queue_url, ReceiptHandle=receipt_handle
        )
        return response

    new_sqs_client = get_aws_sqs_client(credentials=sqs_credentials)
    response = new_sqs_client.delete_message(
        QueueUrl=sqs_credentials.queue_url, ReceiptHandle=receipt_handle
    )
    return response



from pathlib import Path
from navalmartin_mir_aws_utils import FilePathBatch
from navalmartin_mir_aws_utils import AWSCredentials_S3

AWS_S3_REGION = ""
AWS_S3_BUCKET_NAME = ""
IMAGE_STR_TYPES = (".jpg", ".png", ".jpeg")
AWS_S3_REGION_TO = ""
AWS_S3_BUCKET_NAME_TO = ""


def read_pil_image_to_byte_string(image_path: Path):
    try:
        import PIL
        from PIL import Image
        import io

        image = Image.open(image_path)
        img_byte_arr = io.BytesIO()
        # image.save expects a file-like as a argument
        image.save(img_byte_arr, format=image.format)
        # Turn the BytesIO object back into a bytes object
        imgByteArr = img_byte_arr.getvalue()
        return imgByteArr
    except ModuleNotFoundError as e:
        print(f"ERROR: This example needs module {str(e)}")
    except PIL.UnidentifiedImageError as e:
        print(f"ERROR: This does not look like an image file. {str(e)}")
    except Exception as e:
        print(f"{str(e)}")


if __name__ == "__main__":
    aws_s3_credentials = AWSCredentials_S3(
        aws_s3_bucket_name=AWS_S3_BUCKET_NAME, aws_region=AWS_S3_REGION
    )

    image_batch = FilePathBatch(
        s3_credentials=aws_s3_credentials, do_build_client=False
    )

    # set the files from localhost
    image_batch.load_from_list(
        files=[
            "/home/alex/qi3/mir_lambda_tasks/test_images/img_7_8.jpg",
            "/home/alex/qi3/mir_lambda_tasks/test_images/img_7_3.jpg",
            "/home/alex/qi3/mir_lambda_tasks/test_images/not_an_image.txt",
            "/home/alex/qi3/mir_lambda_tasks/test_images/not_an_image.png",
        ]
    )

    print(f"Number of files in batch {len(image_batch)}")
    print(f"The 2nd file is {image_batch[2]}")

    # read the image...
    image_byte = image_batch.read_file_byte_string(
        key=1, read_from_local_host=True, file_reader=read_pil_image_to_byte_string
    )

    # read the image...this is not an image
    image_byte = image_batch.read_file_byte_string(
        key=2, read_from_local_host=True, file_reader=read_pil_image_to_byte_string
    )

    # read the image...this is not an image
    image_byte = image_batch.read_file_byte_string(
        key=3, read_from_local_host=True, file_reader=read_pil_image_to_byte_string
    )

    # read the images
    file_prefix = "6411b6f2a656f521ed52e6cd/0/"
    image_batch.read(
        prefixes=(file_prefix,), valid_image_extensions=IMAGE_STR_TYPES, delimiter="/"
    )

    for image_file in image_batch:
        print(image_file)

    # copy image batch from one bucket
    # to another
    image_batch.reinit(
        s3_credentials=aws_s3_credentials, delimiter="/", do_build_client=True
    )

    image_batch.read(
        prefixes=("6411b6f2a656f521ed52e6cd/0/64120f5a8a32794f722abdb9.jpg",),
        valid_image_extensions=IMAGE_STR_TYPES,
    )

    aws_s3_credentials_to = AWSCredentials_S3(
        aws_s3_bucket_name=AWS_S3_BUCKET_NAME_TO, aws_region=AWS_S3_REGION_TO
    )

    print(f"Copy file batch to: {aws_s3_credentials_to.aws_s3_bucket_name}")
    kwargs = {}
    copy_response = image_batch.copy_file_to(
        key=0, s3_credentials_to=aws_s3_credentials_to, **kwargs
    )
    print(copy_response)

    etag = copy_response["CopyObjectResult"]["ETag"]

    # try to copy if the Etag does not exist
    # this will fail
    kwargs = {"CopySourceIfNoneMatch": etag}
    copy_response = image_batch.copy_file_to(
        key=0, s3_credentials_to=aws_s3_credentials_to, **kwargs
    )



import json
from typing import Any, Callable, Union, Dict
from navalmartin_mir_aws_utils import AWSCredentials_SecretsManager
from navalmartin_mir_aws_utils import get_aws_client_factory

OK = "OK"


def get_error_return(error: str) -> dict:
    return {"result": "FAILED", "error": error}


def get_success_return(success_msg: str = "OK") -> dict:
    return {"result": success_msg, "error": []}


def get_secrets(credentials: AWSCredentials_SecretsManager) -> dict:
    """Returns a dictionary with the SecretString of the specified
    secrets manager
    """
    client = get_aws_client_factory(credentials=credentials)
    get_secret_value_response = client.get_secret_value(
        SecretId=credentials.secret_name
    )
    secret_string = get_secret_value_response["SecretString"]
    secret_string = json.loads(secret_string)
    return secret_string


async def validate_sqs_event_record(
    event: dict, context: Any, db_writer: Callable = None
) -> Union[Dict, str]:
    """Validate the event and write in the DB that monitors the errors

    Parameters
    ----------
    event: The event the lambda was called to validate
    db_writer: The writer to write in the DB
    context: The context the lambda was called

    Returns
    -------

    """
    if "Records" not in event:
        print(f"No 'Records' was provided in the given event. Finishing task...")

        if db_writer is not None:
            await db_writer(
                error_type="GENERAL_SQS_MESSAGE_ERROR",
                error_msg="No 'Records' was provided in the given event",
                task_id=context.function_name,
                lambda_arn=context.invoked_function_arn,
                lambda_request_id=context.aws_request_id,
                event=event,
            )
        return get_error_return(error=f"No 'Records' was provided in the given event.")
    else:
        if len(event["Records"]) == 0:
            print(
                f"'Records' attribute is empty in the provided event. Finishing task..."
            )
            if db_writer is not None:
                await db_writer(
                    error_type="GENERAL_SQS_MESSAGE_ERROR",
                    error_msg="'Records' attribute is empty in the provided event.",
                    task_id=context.function_name,
                    lambda_arn=context.invoked_function_arn,
                    lambda_request_id=context.aws_request_id,
                    event=event,
                )
            return get_error_return(
                error=f"'Records' attribute is " f"empty in the provided event."
            )

        if "body" not in event["Records"][0]:
            print(
                f"'Records' attribute does not have 'body' attribute "
                f"in the provided event. Finishing task..."
            )

            if db_writer is not None:
                await db_writer(
                    error_type="GENERAL_SQS_MESSAGE_ERROR",
                    error_msg="'Records' attribute does not have 'body' "
                    "attribute in the provided event.",
                    task_id=context.function_name,
                    lambda_arn=context.invoked_function_arn,
                    lambda_request_id=context.aws_request_id,
                    event=event,
                )

            return get_error_return(
                error=f"'Records' attribute does not "
                f"have 'body' attribute in the provided event."
            )

        if "receiptHandle" not in event["Records"][0]:
            print(
                f"'Records' attribute does not have 'receiptHandle' "
                f"attribute in the provided event."
            )
            if db_writer is not None:
                await db_writer(
                    error_type="GENERAL_SQS_MESSAGE_ERROR",
                    error_msg="'Records' attribute does not have 'receiptHandle' "
                    "attribute in the provided event.",
                    task_id=context.function_name,
                    lambda_arn=context.invoked_function_arn,
                    lambda_request_id=context.aws_request_id,
                    event=event,
                )
            return get_error_return(
                error=f"'Records' attribute does not have 'receiptHandle' "
                f"attribute in the provided event."
            )

        return OK



"""
Various utilities for working with AWS in Python. The official
PyPi package can be found <a href="https://pypi.org/project/navalmartin-mir-aws-utils/">here</a>.
This package is basically wrappers on top of boto3 that allows for easier development with AWS.
"""
__version__ = "0.0.33"
from .boto3_client import get_aws_client_factory
from .aws_credentials import (
    AWSCredentials_S3,
    AWSCredentials_SQS,
    AWSCredentials_CognitoIDP,
    AWSCredentials_SecretsManager,
    AWSCredentials_SES,
    AWSCredentials_SFN,
    AWSCredentials_SageMaker
)
from .file_s3_batch import FilePathBatch
from .s3_utils import (get_s3_iterator,
                       expand_s3_iterator_contents,
                       expand_s3_iterator_common_prefixes,
                       delete_s3_all_objs_with_key,
                       delete_s3_object_with_key,
                       save_object_to_s3,
                       generate_presigned_url)

from .utils.simple_email import (Email,
                                 EmailBody,
                                 EmailSubject)

from .tasks.ses_tasks.ses_tasks import (verify_email_identity,
                                        send_simple_email,
                                        send_html_email)
